@@ -32,7 +32,7 @@
 #define _GLIBCXX_CXX_CONFIG_H 1
 
 // The current version of the C++ library in compressed ISO date format.
-#define __GLIBCXX__ 20120301
+#define __GLIBCXX__ 20130412
 
 // Macros for various attributes.
 //   _GLIBCXX_PURE
@@ -497,7 +497,7 @@ namespace std
 #define _GLIBCXX_HAVE_EIDRM 1
 
 /* Define to 1 if you have the <endian.h> header file. */
-/* #undef _GLIBCXX_HAVE_ENDIAN_H */
+#define _GLIBCXX_HAVE_ENDIAN_H 1
 
 /* Define if ENODATA exists. */
 /* #undef _GLIBCXX_HAVE_ENODATA */
@@ -868,7 +868,7 @@ namespace std
 #define _GLIBCXX_HAVE_WCTYPE_H 1
 
 /* Define if writev is available in <sys/uio.h>. */
-/* #undef _GLIBCXX_HAVE_WRITEV */
+#define _GLIBCXX_HAVE_WRITEV 1
 
 /* Define to 1 if you have the `_acosf' function. */
 /* #undef _GLIBCXX_HAVE__ACOSF */
@@ -1125,6 +1125,9 @@ namespace std
 /* Define if a fully dynamic basic_string is wanted. */
 /* #undef _GLIBCXX_FULLY_DYNAMIC_STRING */
 
+/* Define if gthreads types cannot be copy-assigned in C++11. */
+/* #undef _GLIBCXX_GTHREADS_NO_COPY_ASSIGN_IN_CXX11 */
+
 /* Define if gthreads library is available. */
 #define _GLIBCXX_HAS_GTHREADS 1
 
